import React from "react";
import { ItemOptions } from "./ReactTreeListItem";
import { ReactTreeListItemType } from "./types/ItemTypes";
export interface ReactTreeListProps {
    /**
     * The data to display in the list.
     */
    data: ReactTreeListItemType[];
    /**
     * Enable dragging of the items
     */
    draggable?: boolean | true;
    /**
     * Function that is triggered when data changes
     */
    onChange(data: ReactTreeListItemType[]): void;
    /**
     * The ID of the selected item
     */
    selectedId?: string | "";
    /**
     * Function that is triggered when an item is selected
     */
    onSelected?(item: ReactTreeListItemType): void;
    /**
     * Function that is triggered when drag completed
     */
    onDrop?(draggingNode: ReactTreeListItemType, dropNode: ReactTreeListItemType, dropType: string): void;
    /**
     * Defines the default values for item object
     *
     * Eg. `itemDefaults={{ open: true }}` will make all items open by default unless specified otherwise
     * inside each item separately.
     */
    itemDefaults?: Partial<Omit<ReactTreeListItemType, "id">>;
    /**
     * Options for the items
     */
    itemOptions?: ItemOptions;
}
export declare const ReactTreeList: React.FC<ReactTreeListProps>;
