"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactTreeList = void 0;
var react_1 = __importStar(require("react"));
var styled_components_1 = __importDefault(require("styled-components"));
var lodash_1 = require("lodash");
var ReactTreeListItem_1 = require("./ReactTreeListItem");
var useUniqueId_1 = require("./hooks/useUniqueId");
var useGetItemById_1 = require("./utils/useGetItemById");
var useUpdateItemById_1 = require("./utils/useUpdateItemById");
var useUpdateSelectedItemById_1 = require("./utils/useUpdateSelectedItemById");
var ReactTreeList = function (_a) {
    var draggable = _a.draggable, data = _a.data, selectedId = _a.selectedId, onChange = _a.onChange, onSelected = _a.onSelected, onDrop = _a.onDrop, itemDefaults = _a.itemDefaults, _b = _a.itemOptions, itemOptions = _b === void 0 ? {} : _b;
    var generateUniqueId = (0, useUniqueId_1.useUniqueId)().generate;
    var lastOpenState = (0, react_1.useRef)(false);
    var getItemById = (0, useGetItemById_1.useGetItemById)(data);
    var updateItemById = (0, useUpdateItemById_1.useUpdateItemById)(data, onChange);
    var _c = (0, react_1.useState)(selectedId), currentSelectedId = _c[0], setCurrentSelectedId = _c[1];
    var updateSelectedItemById = (0, useUpdateSelectedItemById_1.useUpdateSelectedItemById)(data, onChange);
    /**
     * To make sure the event runs only once, we store in this variable
     * whether the event should run.
     */
    var triggerOnChange = false;
    var selectedOnChange = false;
    var removeByIdWithoutOnChange = function (id) {
        var returnItem = undefined;
        var recursiveRemoveId = function (item, index, array) {
            if (returnItem)
                return;
            if (item.id === id) {
                returnItem = item;
                array.splice(index, 1);
                return;
            }
            if (item.children) {
                item.children.forEach(recursiveRemoveId);
            }
        };
        data.forEach(recursiveRemoveId);
        return returnItem;
    };
    var selectedNode = function (item) {
        selectedOnChange = true;
        setCurrentSelectedId(item.id || "");
        if (onSelected) {
            onSelected((0, lodash_1.cloneDeep)(item));
        }
    };
    var moveIdTo = function (id, toId) {
        var copyOfItem = removeByIdWithoutOnChange(id);
        if (!copyOfItem)
            return;
        var item = getItemById(toId);
        if (item) {
            item.open = true;
            if (!item.children) {
                item.children = [copyOfItem];
            }
            else {
                item.children.unshift(copyOfItem);
            }
            triggerOnChange = true;
            if (onDrop && item) {
                var dragingNode = (0, lodash_1.cloneDeep)(copyOfItem);
                if ("children" in dragingNode) {
                    delete dragingNode.children;
                }
                var dragNode = (0, lodash_1.cloneDeep)(item);
                if ("children" in dragNode) {
                    delete dragNode.children;
                }
                onDrop(dragingNode, dragNode, "inner");
            }
        }
    };
    var moveIdBefore = function (id, beforeId) {
        var copyOfItem = removeByIdWithoutOnChange(id);
        var dragNode = null;
        var breakRecursion = false;
        if (!copyOfItem)
            return;
        var recursiveMoveIdAfter = function (item, index, array) {
            if (breakRecursion)
                return;
            if (item.id === beforeId) {
                array.splice(index, 0, copyOfItem);
                breakRecursion = true;
                dragNode = (0, lodash_1.cloneDeep)(item);
            }
            else if (item.children) {
                item.children.forEach(recursiveMoveIdAfter);
            }
        };
        data.forEach(recursiveMoveIdAfter);
        triggerOnChange = true;
        if (onDrop && dragNode) {
            var dragingNode = (0, lodash_1.cloneDeep)(copyOfItem);
            if ("children" in dragingNode) {
                delete dragingNode.children;
            }
            if ("children" in dragNode) {
                delete dragNode.children;
            }
            onDrop(dragingNode, dragNode, "before");
        }
    };
    var moveIdAfter = function (id, afterId) {
        var copyOfItem = removeByIdWithoutOnChange(id);
        var dragNode = null;
        var breakRecursion = false;
        if (!copyOfItem)
            return;
        var recursiveMoveIdAfter = function (item, index, array) {
            if (breakRecursion)
                return;
            if (item.id === afterId) {
                array.splice(index + 1, 0, copyOfItem);
                breakRecursion = true;
                dragNode = (0, lodash_1.cloneDeep)(item);
            }
            else if (item.children) {
                item.children.forEach(recursiveMoveIdAfter);
            }
        };
        data.forEach(recursiveMoveIdAfter);
        triggerOnChange = true;
        if (onDrop && dragNode) {
            var dragingNode = (0, lodash_1.cloneDeep)(copyOfItem);
            if ("children" in dragingNode) {
                delete dragingNode.children;
            }
            if ("children" in dragNode) {
                delete dragNode.children;
            }
            onDrop(dragingNode, dragNode, "after");
        }
    };
    var renderContent = function () {
        /**
         * The children will be rendered as flat, contrary to the tree
         * structure of data.
         */
        var children = [];
        /**
         * A counter for the indentation of items
         */
        var indent = 0;
        var renderItem = function (listItem, index, array, parentOpen, isFirstLoop) {
            var isFirstItemInFirstLoop = isFirstLoop && index === 0;
            if (!listItem.id) {
                triggerOnChange = true;
                array[index].id = generateUniqueId();
            }
            var item = array[index];
            if (parentOpen) {
                children.push(react_1.default.createElement(ReactTreeListItem_1.ReactTreeListItem, { key: item.id, draggable: draggable !== null && draggable !== void 0 ? draggable : true, selectedId: currentSelectedId || "", item: __assign(__assign({}, itemDefaults), item), options: itemOptions, indent: indent, allowDropBefore: isFirstItemInFirstLoop, onSelected: function () {
                        updateSelectedItemById(item.id);
                        selectedNode(item);
                    }, onArrowClick: function () { return updateItemById(item.id, { open: !item.open }); }, onDragging: function (drag) {
                        if (drag) {
                            lastOpenState.current = !!item.open;
                            updateItemById(item.id, { open: false });
                        }
                        else {
                            updateItemById(item.id, { open: lastOpenState.current });
                        }
                    }, onDropInside: function (id, toId) { return moveIdTo(id, toId); }, onDropBefore: function (id, beforeId) { return moveIdBefore(id, beforeId); }, onDropAfter: function (id, afterId) { return moveIdAfter(id, afterId); } }));
            }
            if (item.children) {
                // Indent up before processing children
                indent += 1;
                item.children.forEach(function (nestedListItem, nestedIndex, nestedArray) {
                    return renderItem(nestedListItem, nestedIndex, nestedArray, parentOpen ? item.open : false);
                });
                // Indent down after children processed
                indent -= 1;
            }
        };
        data.forEach(function (listItem, index, array) {
            return renderItem(listItem, index, array, true, true);
        });
        return children;
    };
    (0, react_1.useEffect)(function () {
        if (selectedId) {
            var selectedItem = getItemById(selectedId);
            if (selectedItem) {
                updateSelectedItemById(selectedItem.id);
                selectedNode(selectedItem);
            }
        }
    }, [selectedId]);
    (0, react_1.useEffect)(function () {
        if (triggerOnChange) {
            onChange((0, lodash_1.cloneDeep)(data));
        }
    }, [triggerOnChange]);
    return react_1.default.createElement(Root, null, renderContent());
};
exports.ReactTreeList = ReactTreeList;
var Root = styled_components_1.default.div(templateObject_1 || (templateObject_1 = __makeTemplateObject([""], [""])));
var templateObject_1;
//# sourceMappingURL=ReactTreeList.js.map